import jax.numpy as jnp

def com_1D(positions, masses=None, axis=0):

    if masses is None:
        masses = jnp.ones((len(positions)))

    if positions.ndim == 2:
        xs = positions[:,axis]
    else:
        raise Exception('Only implemented for 2d data')

    return jnp.sum(xs * masses) / len(xs)

def vel_disp_1D(velocities, axis =0):
    
    if velocities.ndim == 3:
        vxs = velocities[:,1,axis]
    else:
        raise Exception('Only implemented for 3d data')
    
    return jnp.std(vxs)




import numpy as np

def make_pos_data(seed=0):
    np.random.seed(seed)
    return np.random.uniform(-1, 1, size=(100, 3))

def make_posvel_data(seed=0):
    np.random.seed(seed)
    return np.random.uniform(-1, 1, size=(100, 2, 3))

def np_bootstrap(data,statistic_func,n_resamples=1000,seed=0):
    """
    Bootstrap method using numpy and a for loop

    Input: 
    data            (array-like)
    statistic_func  (function)
    n_samples       (int)
    seed            (int)
    """

    est_arr = np.zeros(n_resamples)
    np.random.seed(seed)
 
    for i in range(n_resamples):
        idx = np.random.randint(len(data), size=len(data))
        resample = data[idx]
        
        est_arr[i] = statistic_func(resample)

    return est_arr

if __name__=='__main__':
    pass


import jax
import jax.numpy as jnp
import numpy as np

def bootstrap(data, statistic_func, n_resamples, seed):
    resamples = bootstrap_resamples(data, n_resamples, seed)
    est_arr = _apply_vect_statistic(statistic_func, resamples)
    return est_arr

def jackknife(data, statistic_func):
    resamples = jackknife_resamples(data)
    est_arr = _apply_vect_statistic(statistic_func, resamples)
    return est_arr

def _apply_vect_statistic(statistic_func, resamples):
    statistic_vectorized = jax.vmap(statistic_func)
    return statistic_vectorized(resamples)


# def jaxstrapknife(data, statistic_func, n_resamples=None, seed=0):
#     """
#     Bootstrap method using JAX arrays
#     Input: 
#     data            (array-like)
#     statistic_func  (function)
#     n_samples       (int)
#     seed            (int)
#     """
#     if n_resamples is None:
#         resamples = jackknife_resamples(data)
#     else:
#         resamples = bootstrap_resamples(data, n_resamples, seed)
#     statistic_vectorized = jax.vmap(statistic_func)
#     est_arr = statistic_vectorized(resamples)

    # return est_arr


def bootstrap_resamples(data, n_resamples, seed):
    key = jax.random.PRNGKey(seed)
    resamples = jax.random.choice(key,data,(n_resamples,*data.shape),replace=True)
    return resamples


def jackknife_resamples(data):
    redata = jnp.repeat(jnp.array([data]), data.shape[0], axis=0)
    mask = jnp.repeat(~jnp.eye(data.shape[0], dtype=bool).reshape(data.shape[0], data.shape[0], -1), data.shape[-1], axis=2)
    return redata[mask].reshape(data.shape[0], data.shape[0]-1, -1)



from setuptools import setup, find_packages
import re

# copied from orbitize
def get_requires():
    reqs = []
    for line in open('requirements.txt', 'r').readlines():
        reqs.append(line)
    return reqs

# also from orbitize, auto-updating version code stolen from RadVel
def get_property(prop, project):
    result = re.search(
        r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format(prop),
        open(project + "/__init__.py").read(),
    )
    return result.group(1)

setup(
    name = "jaxknife",
    version=get_property("__version__", "jaxknife"),
    packages = find_packages(),
    install_requires = get_requires()
)